/*
 *                                 jStock
 *                    (C) Copyright 2003 - 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * ChartPanel.java
 *
 * Created on 06 December 2001, 22:02
 */

package com.exponto.jstock;

import java.util.ArrayList;
import javax.swing.BoxLayout;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.Graphics;
import java.awt.Graphics2D;
import javax.swing.JPanel;
import javax.swing.event.MouseInputListener;
import java.awt.event.KeyEvent;
import org.apache.log4j.Logger;

//import com.exponto.jstock.events.UpdateDisplayListener;
//import com.exponto.jstock.events.UpdateDisplayEvent;
import com.exponto.jstock.indicators.Indicator;
import com.exponto.jstock.popupwindows.*;


/**
 * Responsibility of this class is to decide what panels are needed and to calculate their size and
 * posistion. Also loads the intial dataset, but further processing of that may be required by the 
 * "drawing" classes.
 *
 *
 * @author  kaare
 * @version 
 */

public class ChartPanel extends JPanel{
    private static Dimension currentDim = new Dimension();
    private static Dimension indicatorPanelDim = new Dimension();
    private static Dimension candleDim= new Dimension();
    private static UserPreferences userPrefs = UserPreferences.getInstance();
	private static Notes notes;   
    private String ticker;
	private CandleStickPanel candleStick;
    private static GraphicsParameters graphicsParameters = new GraphicsParameters();
    private ChartData cD;
	private static ArrayList indicatorList;
	private JPanel [] indicatorPanelsArray;
	static Logger logger = Logger.getLogger(ChartPanel.class);

    double x1 = 0.0;
    double x2 = 0.0;
    double y1 = 0.0;
    double y2 = 0.0;
    boolean newLine = true;
	int highLightedStickNumber = -1;
    /** Creates new ChartPanel */
    public ChartPanel(){
    }

    public ChartPanel(String tick) throws TickerNotFoundException {
    	ticker = tick;
        cD = new ChartData(ticker);
        //Change if other datasource than file is required
        String fname = userPrefs.getFilePath() + ticker + "." + userPrefs.getExtension();
//        logger.info("Loading file: " + fname);
        cD.loadLogDataSeries(fname, userPrefs.getNumberOfDays());
        //currentDim.setSize(dim.getWidth() -1, dim.getHeight() - 40);
        this.setSize(currentDim);
        // For daily panels
        makePanels(false);
		setVisible(true);        
		//pack();
	}
 
	public ChartPanel(Boolean b){
	 }

	public ChartPanel(ChartData chData) {
//		Think this is a hack		
		cD = chData;
		this.setSize(currentDim);
		// This is for weekly panels
		makePanels(true);
		setVisible(true);
	}

	public static void setAvailableSize(Dimension dim) {
		//Does not correct the size if more than one rows of tabs are added.
		// see initComponents() in jStock for further comments on tabs. 
		currentDim.setSize(dim.getWidth() - 5, dim.getHeight() - 27);
		//logger.debug("currentDim set by jStock: "+ currentDim.toString());
		updatePanelsSize();
		CandleStickPanel.setAvailableSize(candleDim);
	}
	
	public ChartData getChartData() {
		return cD;
	}

	private void makePanels(boolean weekly) {
		LogDataSeries lDS;
		if (weekly){
			lDS = cD.getWeeklyLogDataSeries();
		} else {
			lDS = cD.getLogDataSeries();
		}
		indicatorList = updatePanelsSize();
		graphicsParameters.setDimension(indicatorPanelDim);
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		candleStick = new CandleStickPanel(lDS);
		candleStick.setPreferredSize(candleDim);
		mouseInput();
		keyInput();
		this.add(candleStick);
		indicatorPanelsArray = new IndicatorPanel [indicatorList.size()];
		for (int i = 0; i < indicatorList.size(); i++) {
			Indicator ind = (Indicator) indicatorList.get(i);
//			Create separate IndicatorPanel for graphs
			IndicatorPanel indicatorPanel = new IndicatorPanel(lDS, ind);
			indicatorPanelsArray[i] = indicatorPanel;
			indicatorPanel.setGraphicsParameters(graphicsParameters);
			this.add(indicatorPanel);
//			logger.debug("Added Indicator panel number: " + i);
		}
			//logger.info("Class ChartPanel, rsiPanel not null and added to layoutmanager");
		if (userPrefs.getNotes()) {
			if (notes == null) {
				notes = Notes.getInstance(cD.getTickerId());
			} else {
				notes.refresh(cD.getTickerId());
			}
		}
	}
 
	private static ArrayList updatePanelsSize() {
		double scaling;
		int minChartHeight = 40;
		int defaultIndicatorPanelHeight = 15;
		String prop1 = "Scaling.MinimumCandlestickHeight";
		String prop2 = "Scaling.DefaultIndicatorPanelHeight";
		String csHeight = userPrefs.getProperty(prop1);
		String iPHeight = userPrefs.getProperty(prop2);
		if (csHeight == null) {
			userPrefs.setProperty(prop1, Integer.toString(minChartHeight));
		} else {
			try {
				minChartHeight = Integer.parseInt(csHeight) ;            
			} catch (NumberFormatException e) {
				logger.error("Got a NumberFormatException when trying to read Scaling.MinimumCandlestickHeight from file, properties file probably corrupt");
			}
		}
		if (iPHeight == null) {
			userPrefs.setProperty(prop2, Integer.toString(defaultIndicatorPanelHeight));
		} else {
			try {
				defaultIndicatorPanelHeight = Integer.parseInt(iPHeight) ;            
			} catch (NumberFormatException e) {
				logger.error("Got a NumberFormatException when trying to read Scaling.DefaultIndicatorPanelHeight from file, properties file probably corrupt");
			}
		}
		
		IndicatorController indController = IndicatorController.getInstance();
		int numberOfPanels = indController.numberOfPanels();
		logger.debug("Number of panels is " + numberOfPanels);
		if (numberOfPanels < 1) {
			scaling = 0;
		} else if ((100 - numberOfPanels * defaultIndicatorPanelHeight) < minChartHeight ) {
			scaling = (double) (100 - minChartHeight) / numberOfPanels;
		} else {
			scaling = (double) numberOfPanels * defaultIndicatorPanelHeight/100;
			logger.debug("Default scaling");
		}
		//logger.debug("Scaling is: " + scaling);
		indicatorPanelDim.setSize(currentDim.getWidth(), (currentDim.getHeight()) * (scaling) / numberOfPanels);
//		logger.debug(indicatorPanelDim.toString());
		candleDim.setSize(currentDim.getWidth(), (currentDim.getHeight())  * (1 - scaling));
//		logger.debug("CurrentDim: " + currentDim.toString());
//		logger.debug("CandleDim: " + candleDim.toString());
//		logger.debug("indicatorPanelDim: " + indicatorPanelDim.toString());
		graphicsParameters.setDimension(indicatorPanelDim);
		return indController.getEnabledIndicators();
	}
    
	private void keyInput()  {
		/**
		 * 
		 */
		KeyListener kL = new KeyListener() {
		
			public void keyPressed(KeyEvent e) {
				//char ch = e.getKeyChar();
				int keyCode = e.getKeyCode();
				if (userPrefs.getDrawMode()) {
					updateIndicatorStatus(indicatorPanelsArray, -1);
					candleStick.keyPressed(keyCode);
				} else {
					if (keyCode == KeyEvent.VK_RIGHT) {
						highLightedStickNumber++;
						candleStick.updateStickNumber(highLightedStickNumber);
						updateIndicatorStatus(indicatorPanelsArray, highLightedStickNumber);
					} else if (keyCode == KeyEvent.VK_LEFT) {
						highLightedStickNumber--;
						candleStick.updateStickNumber(highLightedStickNumber);
						updateIndicatorStatus(indicatorPanelsArray, highLightedStickNumber);
					}
				}
				//System.out.println(" Key pressed " + e.getKeyCode());
			}

			public void keyReleased(KeyEvent e) {
				//candleStick.keyReleased(e.getKeyCode());
	//			System.out.println(" Key released");
			}
	
			/* (non-Javadoc)
			 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
			 */
			public void keyTyped(KeyEvent e) {
				//candleStick.keyTyped(e.getKeyChar());
				char ch = e.getKeyChar();
				if (userPrefs.getDrawMode()) {
					updateIndicatorStatus(indicatorPanelsArray, -1);
					candleStick.keyTyped(e.getKeyChar());
				} else {
					if (ch == 'n') {
//						Next
						highLightedStickNumber++;
						candleStick.updateStickNumber(highLightedStickNumber);
						updateIndicatorStatus(indicatorPanelsArray, highLightedStickNumber);
					} else if (ch == 'p') {
						// Previous
						highLightedStickNumber--;
						candleStick.updateStickNumber(highLightedStickNumber);
						updateIndicatorStatus(indicatorPanelsArray, highLightedStickNumber);
					}
				}
				//System.out.println(" Key typed " + e.getKeyChar());
			}
		};
		candleStick.addKeyListener(kL);
		add(candleStick);
		setVisible(true);
		candleStick.repaint();    	
	}
  
    private void mouseInput() {
		MouseInputListener mil = new MouseInputListener() {
			public void mousePressed(MouseEvent e) {
				if (userPrefs.getDrawMode()) {
					highLightedStickNumber = -1;
					candleStick.mousePress(e.getX(), e.getY());
					candleStick.updateStickNumber(highLightedStickNumber);
					updateIndicatorStatus(indicatorPanelsArray, highLightedStickNumber);
				}
				//System.out.println("mousePressed");
			}
			public void mouseDragged(MouseEvent e) {
				//System.out.println("mouseDragged");
				// Button down
				candleStick.mouseDrag(e.getX(), e.getY());
			}
			public void mouseMoved(MouseEvent e) {
				
				// No button down
				//System.out.println("mouseMoved");
			}
			public void mouseClicked(MouseEvent e) {
				// Might need something like this in order to get the key input
				//boolean focus = this.requestFocusInWindow(false);
				candleStick.panelRequestFocus();
				if (!userPrefs.getDrawMode()) {
					highLightedStickNumber = (e.getX() - userPrefs.getLMargin()) /userPrefs.getPixelPrStick();
					candleStick.updateStickNumber(highLightedStickNumber);
					updateIndicatorStatus(indicatorPanelsArray, highLightedStickNumber);
				}

				//candleStick.mouseClick(e.getX(), e.getY());
				//System.out.println("Klikk 2");
//				setVisible(true);
//				candleStick.repaint();
			}
			public void mouseReleased(MouseEvent e) {
				
				if (userPrefs.getDrawMode()) {
					highLightedStickNumber = -1;
					candleStick.mouseRelease(e.getX(), e.getY());
					candleStick.updateStickNumber(highLightedStickNumber);
					updateIndicatorStatus(indicatorPanelsArray, highLightedStickNumber);

				}
				//System.out.println("mouseReleased");
			}
			public void mouseEntered(MouseEvent e) {
				candleStick.panelRequestFocus();
			}
			public void mouseExited(MouseEvent e) {
				//System.out.println("mouseExited");
			}
		};
		candleStick.addMouseListener(mil);
		candleStick.addMouseMotionListener(mil);
    }
    public void refresh() {
		if (userPrefs.getNotes()) {
			if (notes == null) {
				notes = Notes.getInstance(cD.getTickerId());
			} else {
				notes.refresh(cD.getTickerId());
			}
		}    	
    }
    
	public void paint(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;
		super.paint(g2d);
	}

	private void updateIndicatorStatus(JPanel [] indList, int number) {
		int dayNumber;
		for (int i = 0; i < indList.length; i++) {
			IndicatorPanel indPanel = (IndicatorPanel) indList[i];
			indPanel.updateDailyPrintout(number);
		}
	}
	
//	public void panelRequestFocus() {
//		boolean focus = requestFocusInWindow(false);
//		//System.out.println("mouseEntered, focus is " + focus);
//		System.out.println("Focus  " + focus);
//	}
	
//		g2d.drawString(" Just a test of drawing from the super class as well, 1, 2, 3 testing ",  100, 100);
// This work good as gold, may use this to move (most of) the mouse part out of the candlestick class
//	private void makeWeeklyPanel() {
//	indicatorList = updatePanelsSize();
//	graphicsParameters.setDimension(indicatorPanelDim);
//	this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
////	This one gets a complete LogDataSeries
//	candleStick = new CandleStickPanel(cD.getWeeklyLogDataSeries());
//	candleStick.setPreferredSize(candleDim);
//	mouseInput();
//	keyInput();
//	this.add(candleStick);
//
//	for (int i = 0; i < indicatorList.size(); i++) {
//		Indicator ind = (Indicator) indicatorList.get(i);
////		Create separate IndicatorPanel for graphs
//		IndicatorPanel indicatorPanel = new IndicatorPanel(cD.getWeeklyLogDataSeries(), ind);
//		indicatorPanel.setGraphicsParameters(graphicsParameters);
//		this.add(indicatorPanel);
////		logger.debug("Added Indicator panel number: " + i);
//	}
//		//logger.info("Class ChartPanel, rsiPanel not null and added to layoutmanager");
//	if (userPrefs.getNotes()) {
//		if (notes == null) {
//			notes = Notes.getInstance(cD.getTickerId());
//		} else {
//			notes.refresh(cD.getTickerId());
//		}
//	}
//}
}
