/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * UserPreferences.java
 *
 * Created on 24 February 2003, 22:52
 *
 * Replaces Chartinfo file
 */

package com.exponto.jstock;
import java.util.*;
import java.io.*;
import java.awt.*;
import java.lang.System;
import org.apache.log4j.Logger;


/**
 *	Takes care of the properties file. It is terrible long and rather boring
 * to update, but not much exiting goes on here. It is just needed to
 * get data in and out of the properties file.
 *
 * Note that this is a singleton object in order to prevent that several
 * instances is used/updated at the same time, use getInstance() to get a 
 * the instance
 * 
 * @author  kaare
 */

public class UserPreferences {
	static Logger logger = Logger.getLogger(UserPreferences.class);
    static private UserPreferences instance; // THE instance of this class
    private Properties prop;
    private String fname;
    
    private Dimension userDim = new Dimension();
    // Defines names for all properties
    static private String filePath = "Directory for datafiles";
    static private String extension = "Default file extension";
    static private String xSize = "Display length";
    static private String ySize = "Display height";
    static private String xOffset = "X offset";
    static private String yOffset = "Y offset";
    static private String lMargin = "Left margin (pixels)";
    static private String rMargin = "Right margin (pixels)";
    static private String tMargin = "Top margin (pixels)";
    static private String bMargin = "Bottom margin (pixels)";
	static private String drawMode = "Draw mode";
	static private String showWeekly = "Show weekly charts";
	static private String notes = "Window for notes open or not";
    static private String pixelPrStick = "Pixels per stick";
//    static private String rsiDays = "RSI periode (days)";
	static private String maDays = "Moving Average periode (days)";
	static private String minorLineLevel = "Minor Support Line Level";
//    static private String scalingFactorChart = "Chart scaling factor";
    static private String scalingFactorChartVolume = "Volume scaling factor";
//    static private String scalingFactorRsi = "RSI scaling factor";
    static private String numberOfDays = " Number of days";  // The max number of days of series that is kept in memory
	static private String exchange = "Exchange";
	static private String enableDB = "Enable db support";
	static private String jdbcDriver = "JBDC Driver";
	static private String dbHost = "DB Host";
	static private String dbName = "DB Name";
	static private String dbUser = "DB User";
	static private String dbPW = "DB Password";
	static private String enablePos = "Enable colum selection";
	static private String tickerPos = "Position of ticker colum";
	static private String exchPos = "Position of exchange colum";
	static private String datePos = "Position of date colum";
	static private String dateFormat = "Format of date field";
	static private String openPos = "Position of open colum";
	static private String highPos = "Position of high colum";
	static private String lowPos = "Position of low colum";
	static private String closePos = "Position of close colum";
	static private String volumePos = "Position of volume colum";
	
     /** Creates a new instance of UserPreferences */
    private UserPreferences() {
        fname = "jStock.properties";
        checkForPropertiesFiles();
        loadProperties();
    }
    private UserPreferences(String fn) {
        fname = fn;
        loadProperties();
        //System.out.println("Properties ok");
    }
            
    private void loadProperties() {
        // Setting application default values before loading user default
        prop = new Properties();
        prop.setProperty(filePath, "/data/");
        prop.setProperty(extension, "php");
        prop.setProperty(xSize, "800");
        prop.setProperty(ySize, "600");
        prop.setProperty(xOffset, "50");
        prop.setProperty(yOffset, "50");
        prop.setProperty(lMargin, "70");
        prop.setProperty(rMargin, "30");
        prop.setProperty(tMargin, "20");
        prop.setProperty(bMargin, "50");
		prop.setProperty(showWeekly, "false");
		prop.setProperty(drawMode, "false");
		prop.setProperty(notes, "false");
        prop.setProperty(pixelPrStick, "4");
		prop.setProperty(maDays, "0");
		prop.setProperty(minorLineLevel, "60");
        prop.setProperty(scalingFactorChartVolume, "0.2");
		prop.setProperty(numberOfDays, "1200");
		prop.setProperty(exchange, "ose");
		prop.setProperty(enableDB, "false");
		prop.setProperty(jdbcDriver, "org.postgresql.Driver");
		prop.setProperty(dbHost, "brand");
		prop.setProperty(dbName, "jstock");
		prop.setProperty(dbUser, "jstock");
		prop.setProperty(dbPW, "jstock");
		prop.setProperty(enablePos, "false");
		prop.setProperty(tickerPos, "0");
		prop.setProperty(exchPos, "0");
		prop.setProperty(datePos, "0");
		prop.setProperty(dateFormat, "yyyyMMdd");
		prop.setProperty(openPos, "0");
		prop.setProperty(highPos, "0");
		prop.setProperty(lowPos, "0");
		prop.setProperty(closePos, "0");
		prop.setProperty(volumePos, "0");
		//       prop.setProperty(rsiDays, "14");
		//       prop.setProperty(scalingFactorRsi, "0.15");		
//		prop.setProperty(scalingFactorChart, "1");		
        loadUserPref();
    }
    
    static public UserPreferences getInstance() {
        if (instance==null) {
            instance = new UserPreferences();
        }
        return instance;
    }
/**
 * Check for existing properties files, first in <userhome>/jstock then in the 
 * application directory. If non exists, it will created the <userhome>/jstock directory
 * if it does not exist.   
 *
 * Upon exit the application will then try to save the properties file in that directory.
 */
	private void checkForPropertiesFiles() {
		String jstockHome = System.getProperty("user.home") + "/jstock/";
		File propertiesFile = new File(jstockHome+fname);
		//logger.debug("jstockHome is: " + jstockHome);
		if (propertiesFile.isFile()) {
			logger.debug("Found properties file at " + jstockHome+fname);
			fname = jstockHome+fname;
		} else {
			propertiesFile = new File(fname);
			if (propertiesFile.isFile()) {
				logger.debug("Found properties file at alternate location " + fname);
			} else {
				propertiesFile = new File(jstockHome);
				fname = jstockHome+fname;
				if (propertiesFile.exists()) {
					logger.info("No properties file is found, but it will be automatically created upon exit");
				} else if (!propertiesFile.mkdir()) {
					logger.error("No properties file found and could not create new directory for properties file at " + jstockHome);
				}
				
			}
		}
		File testFile = new File(jstockHome+fname);
		try {
			//TODO: Bug - trying to create new file even if one already exists
			testFile.createNewFile();
		} catch (IOException e) {
			logger.error(" Failed to create " + jstockHome+fname + " due to " +e.toString());
		}
		
	}
	
	public String getProperty(String name) {
		return prop.getProperty(name);
	}
	
	public void setProperty(String name, String value){
		prop.setProperty(name, value);
	}
	
    public Dimension getUserDim() {
        userDim.setSize(getXSize(), getYSize());
        return userDim;
    }
    public String getFilePath() {
        return prop.getProperty(filePath);
    } 
    public String getExtension() {
        return prop.getProperty(extension);
    }  
    
    public int getLMargin() {
        int i=0;
        try {
            i = Integer.parseInt(prop.getProperty(lMargin)) ;            
        } catch (NumberFormatException e) {
        	logger.error("Got a NumberFormatException when trying to read Left Margin from file, properties file probably corrupt");
            //System.out.println("Class UserPreferences, method getLMargin: NumberFormatException");
        }
        return i;
    }
    
    public int getRMargin() {
        int i=0;
        try {
            i = Integer.parseInt(prop.getProperty(rMargin)) ;            
        } catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read Right Margin from file, properties file probably corrupt");
            //System.out.println("Class UserPreferences, method getRMargin: NumberFormatException");
        }
        return i;
    }

    public int getTMargin() {
        int i=0;
        try {
            i = Integer.parseInt(prop.getProperty(tMargin)) ;            
        } catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read Top Margin from file, properties file probably corrupt");
            //System.out.println("Class UserPreferences, method getTMargin: NumberFormatException");
        }
        return i;
    }
    
    public int getBMargin() {
        int i=0;
        try {
            i = Integer.parseInt(prop.getProperty(bMargin)) ;            
        } catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read Bottom Margin from file, properties file probably corrupt");
            //System.out.println("Class UserPreferences, method getBMargin: NumberFormatException");
        }
        return i;
    }

	public int getMaDays() {
		int i=0;
		try {
			i = Integer.parseInt(prop.getProperty(maDays)) ;            
		} catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read MA days from file, properties file probably corrupt");
		}
		return i;
	}
	public void setMaDays(int i) {
		prop.setProperty(maDays,String.valueOf(i));
	}

	public void setMaDays(String s) {
		prop.setProperty(maDays, s);
	}
	public int getMinorLineLevelDays() {
		int i=0;
		try {
			i = Integer.parseInt(prop.getProperty(minorLineLevel)) ;            
		} catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read Minor Support Line Level from file, properties file probably corrupt");
		}
		return i;
	}
	public void setMinorLineLevel(int i) {
		prop.setProperty(minorLineLevel,String.valueOf(i));
	}

	public void setMinorLineLevel(String s) {
		prop.setProperty(minorLineLevel, s);
	}


	public boolean getDrawMode() {
		String s = prop.getProperty(drawMode);
		if (s.equalsIgnoreCase("false")) return false;
		else return true;
	}
	public void setDrawMode(boolean b) {
		prop.setProperty(drawMode,String.valueOf(b));
	}
    
	public boolean getShowWeekly() {
		String s = prop.getProperty(showWeekly);
		if (s.equalsIgnoreCase("false")) return false;
		else return true;
	}
	public void setShowWeekly(boolean b) {
		prop.setProperty(showWeekly,String.valueOf(b));
	}
	
	public boolean getNotes() {
		String s = prop.getProperty(notes);
		if (s.equalsIgnoreCase("false")) return false;
		else return true;
	}
	public void setNotes(boolean b) {
		prop.setProperty(notes,String.valueOf(b));
	}    
    
    public int getPixelPrStick() {
        int i = 0;
        try {
            i = Integer.parseInt(prop.getProperty(pixelPrStick)) ;            
        } catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read Pixel per stick from file, properties file probably corrupt. Using the default value.");
            //System.out.println("Class UserPreferences, method getPixelPrStick: NumberFormatException");
            // Default value
            return 4;
        } 
        return i;
    }
    public void setPixelPrStick(int i) {
        prop.setProperty(pixelPrStick,String.valueOf(i));
    }
	public void setPixelPrStick(String s) {
		prop.setProperty(pixelPrStick, s);
	}
    public int getYSize() {
        int i = 0;
        try {
            i = Integer.parseInt(prop.getProperty(ySize)) ;            
        } catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read Y Size from file, properties file probably corrupt");
            //System.out.println("Class UserPreferences, method getYSize: NumberFormatException");
        } 
        return i;
    }
    public void setYSize(int i) {
        prop.setProperty(ySize,String.valueOf(i));
    }
    public int getXSize() {
        int i = 0;
        try {
            i = Integer.parseInt(prop.getProperty(xSize)) ;            
        } catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read X size from file, properties file probably corrupt");
            //System.out.println("Class UserPreferences, method getXSize: NumberFormatException");
        }
        return i;
    }
    public void setXSize(int i) {
        prop.setProperty(xSize,String.valueOf(i));
    }    
    public double getScalingFactorChartVolume() {
        double d = 0.0;
        try {
            d = Double.parseDouble(prop.getProperty(scalingFactorChartVolume)) ;            
        } catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read Scaling Factor Chart Volume from file, properties file probably corrupt");
           // System.out.println("Class UserPreferences, method getScalingFactorChartVolume: NumberFormatException");
        }
        return d;
    }
    public void  setScalingFactorChartVolume(double d) {
        prop.setProperty(scalingFactorChartVolume,String.valueOf(d));
    }    
    
	public boolean getEnablePos() {
		String s = prop.getProperty(enablePos);
		if (s.equalsIgnoreCase("false")) return false;
		else return true;
	}
	public void setEnablePos(boolean b) {
		prop.setProperty(enablePos,String.valueOf(b));
	}

    
	public int getTickerPos() {
		int i=0;
		try {
			i = Integer.parseInt(prop.getProperty(tickerPos)) ;            
		} catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read Ticker position from file, properties file probably corrupt");
			//System.out.println("Class UserPreferences, method getRsiDays: NumberFormatException");
		}
		return i;
	}
	public void setTickerPos(int i) {
		prop.setProperty(tickerPos,String.valueOf(i));
	}

	public void setTickerPos(String s) {
		prop.setProperty(tickerPos, s);
	}

	public int getExchangePos() {
		int i=0;
		try {
			i = Integer.parseInt(prop.getProperty(exchPos)) ;            
		} catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read exchange position from file, properties file probably corrupt");
			//System.out.println("Class UserPreferences, method getRsiDays: NumberFormatException");
		}
		return i;
	}
	public void setExchangePos(int i) {
		prop.setProperty(exchPos,String.valueOf(i));
	}

	public void setExchangePos(String s) {
		prop.setProperty(exchPos, s);
	}

	public int getDatePos() {
		int i=0;
		try {
			i = Integer.parseInt(prop.getProperty(datePos)) ;            
		} catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read date position from file, properties file probably corrupt");
			//System.out.println("Class UserPreferences, method getRsiDays: NumberFormatException");
		}
		return i;
	}
	public void setDatePos(int i) {
		prop.setProperty(datePos,String.valueOf(i));
	}

	public void setDatePos(String s) {
		prop.setProperty(datePos, s);
	}

	public String getDateFormat() {
		return prop.getProperty(dateFormat);
	}

	/**
	 * @param string Format of the dates
	 */
	public void setDateFormat(String string) {
		prop.setProperty(dateFormat, string);
	}

	public int getOpenPos() {
		int i=0;
		try {
			i = Integer.parseInt(prop.getProperty(openPos)) ;            
		} catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read open position from file, properties file probably corrupt");
			//System.out.println("Class UserPreferences, method getRsiDays: NumberFormatException");
		}
		return i;
	}
	public void setOpenPos(int i) {
		prop.setProperty(openPos,String.valueOf(i));
	}

	public void setOpenPos(String s) {
		prop.setProperty(openPos, s);
	}

	public int getHighPos() {
		int i=0;
		try {
			i = Integer.parseInt(prop.getProperty(highPos)) ;            
		} catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read high position from file, properties file probably corrupt");
			//System.out.println("Class UserPreferences, method getRsiDays: NumberFormatException");
		}
		return i;
	}
	public void setHighPos(int i) {
		prop.setProperty(highPos,String.valueOf(i));
	}

	public void setHighPos(String s) {
		prop.setProperty(highPos, s);
	}

	public int getLowPos() {
		int i=0;
		try {
			i = Integer.parseInt(prop.getProperty(lowPos)) ;            
		} catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read low position from file, properties file probably corrupt");
			//System.out.println("Class UserPreferences, method getRsiDays: NumberFormatException");
		}
		return i;
	}
	public void setLowPos(int i) {
		prop.setProperty(lowPos,String.valueOf(i));
	}

	public void setLowPos(String s) {
		prop.setProperty(lowPos, s);
	}

	public int getClosePos() {
		int i=0;
		try {
			i = Integer.parseInt(prop.getProperty(closePos)) ;            
		} catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read close position from file, properties file probably corrupt");
			//System.out.println("Class UserPreferences, method getRsiDays: NumberFormatException");
		}
		return i;
	}
	public void setClosePos(int i) {
		prop.setProperty(closePos,String.valueOf(i));
	}

	public void setClosePos(String s) {
		prop.setProperty(closePos, s);
	}

	public int getVolumePos() {
		int i=0;
		try {
			i = Integer.parseInt(prop.getProperty(volumePos)) ;            
		} catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read volume position from file, properties file probably corrupt");
			//System.out.println("Class UserPreferences, method getRsiDays: NumberFormatException");
		}
		return i;
	}
	public void setVolumePos(int i) {
		prop.setProperty(volumePos,String.valueOf(i));
	}

	public void setVolumePos(String s) {
		prop.setProperty(volumePos, s);
	}

   /**
    * Saves all properties to the properties file.
    *
    */
    public void saveAll() {
        saveUserPref();
    }
    private void saveUserPref() {
        try {
            prop.store(new FileOutputStream(fname),fname);
			System.out.println("Saving preferences");
        } catch (IOException e) {
        	logger.error("Cannot save properties file "+ fname);
            //System.out.println("Cannot save properties file "+ fname);
        }
    }
    private void loadUserPref() {
         try {
            prop.load(new FileInputStream(fname));
         } catch (IOException e) {
         	logger.error("Cannot load properties file "+ fname +", continuing with application defaults");
            //System.out.println("Cannot load properties file "+ fname);
            // Write a new file
            saveUserPref();
            //System.out.println("Continuing with application defaults");
            
        }
    }
	/**
	 * @return the number of days to use
	 */
	public int getNumberOfDays() {
		int i = 0;
		try {
			i = Integer.parseInt(prop.getProperty(numberOfDays)) ;            
		} catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read Number of Days from file, properties file probably corrupt");
			//System.out.println("Class UserPreferences, method getNumberOfDays: NumberFormatException");
		}
		return i;
	}

	/**
	 * @return the default exchange name
	 */
	public String getExchange() {
		return prop.getProperty(exchange);
	}

	/**
	 * Set the default exchange
	 * @param string default exchange
	 */
	public  void setExchange(String string) {
		prop.setProperty(exchange, string);
	}


	public boolean getEnableDB() {
		String s = prop.getProperty(enableDB);
		if (s.equalsIgnoreCase("false")) return false;
		else return true;
	}
	public void setEnableDB(boolean b) {
		prop.setProperty(enableDB,String.valueOf(b));
	}

	/**
	 * @return the jdbc driver to use
	 */
	public String getJdbcDriver() {
		return prop.getProperty(jdbcDriver);
	}

	/**
	 * @param string the jdbc driver to set
	 */
	public  void setJdbcDriver(String string) {
		prop.setProperty(jdbcDriver, string);
	}

	/**
	 * @return the db host
	 */
	public String getDbHost() {
		return prop.getProperty(dbHost);
	}

	/**
	 * @return the db name
	 */
	public String getDbName() {
		return prop.getProperty(dbName);
	}

	/**
	 * Security wise this is not a very good idea, but the password is 
	 * stored in clear text.
	 * 
	 * 
	 * @return the db password
	 */
	public String getDbPW() {
		return prop.getProperty(dbPW);
	}

	/**
	 * @return the db user.
	 */
	public String getDbUser() {
		return prop.getProperty(dbUser);
	}

	/**
	 * @param string the db host
	 */
	public void setDbHost(String string) {
		prop.setProperty(dbHost, string);
	}

	/**
	 * @param string the db name
	 */
	public void setDbName(String string) {
		prop.setProperty(dbName, string);
	}

	/**
	 * @param string the db password
	 */
	public void setDbPW(String string) {
		prop.setProperty(dbPW, string);
	}

	/**
	 * @param string the db user
	 */
	public void setDbUser(String string) {
		prop.setProperty(dbUser, string);
	}



	/**
	 * @param string the file extension
	 */
	public void setExtension(String string) {
		prop.setProperty(extension, string);
	}

	/**
	 * @param string the filepath
	 */
	public void setFilePath(String string) {
		prop.setProperty(filePath, string);
	}

//	public double getScalingFactorRsi() {
//		double d = 0.0;
//		try {
//			d = Double.parseDouble(prop.getProperty(scalingFactorRsi)) ;            
//		} catch (NumberFormatException e) {
//		  logger.error("Got a NumberFormatException when trying to read Scaling Factor RSI from file, properties file probably corrupt");			
//			//System.out.println("Class UserPreferences, method getScalingFactorRsi: NumberFormatException");
//		}        
//		return d;
//	}
//	public void  setScalingFactorRsi(double d) {
//		prop.setProperty(scalingFactorRsi,String.valueOf(d));
//	}

//	public int getRsiDays() {
//		int i=0;
//		try {
//			i = Integer.parseInt(prop.getProperty(rsiDays)) ;            
//		} catch (NumberFormatException e) {
//		  logger.error("Got a NumberFormatException when trying to read RSI days from file, properties file probably corrupt");
//			//System.out.println("Class UserPreferences, method getRsiDays: NumberFormatException");
//		}
//		return i;
//	}
//	public void setRsiDays(int i) {
//		prop.setProperty(rsiDays,String.valueOf(i));
//	}
//
//public void setRsiDays(String s) {
//	  prop.setProperty(rsiDays, s);
//}

//public double getScalingFactorChart() {
//	  double d = 0.0;
//	  try {
//		  d = Double.parseDouble(prop.getProperty(scalingFactorChart)) ;            
//	  } catch (NumberFormatException e) {
//		logger.error("Got a NumberFormatException when trying to read Scaling Factor Chart from file, properties file probably corrupt");
//		  //System.out.println("Class UserPreferences, method getScalingFactorChart: NumberFormatException");
//	  }
//	  return d;
//}
//public void  setScalingFactorChart(double d) {
//	  prop.setProperty(scalingFactorChart,String.valueOf(d));
//} 


}
