/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 30-Mar-2004
 *
 */
package com.exponto.jstock.indicators;

//import com.exponto.jstock.Constants;
import com.exponto.jstock.LogDataSeries;
import com.exponto.jstock.LogTradingData;
import java.util.ArrayList;
import org.apache.log4j.Logger;


/**
 * @author kaare
 *
 */
public class EMAAlgorithm {
	static Logger logger = Logger.getLogger(EMAAlgorithm.class);	
	
	/**
	 * Calculates the EMA over a periode. The EMA hve to be scaled if it is 
	 * supposed to be drawn in a logaritmic chart. 
	 * 
	 * @param logDS the logDataSeries 
	 * @param periode the number of days to use in the calculation
	 * @return an ArrayList with EMA(periode) data
	 */
	public static ArrayList computeEMA(LogDataSeries logDS, int periode) {
		ArrayList emaList = new ArrayList();
		LogTradingData lTD;
		Double temp;
		double ema = 0.0;
		double previousEma = 0.0;
		double factor = (double)  2 / (periode +1);
		logDS.setCurrentPeriode();
		if (periode > logDS.logStock.size() - 10) {
			// No meaningful calculation is possible
			return emaList;
		}
		for (int i = 0; i < periode;) {
			// Finds initial EMA (that is moving average for the first days)
			lTD = logDS.getNextLTD();
			if (lTD.isTraded()) {
				previousEma = previousEma + lTD.getClose();
				i++;
			} 
			emaList.add(null);
		}
		previousEma = (double) previousEma / periode;
		// The first numberOfDays values are not correct
		while (logDS.hasNextLTD()) {
			lTD = logDS.getNextLTD();
			if (lTD.isTraded()) {
				//( (Price(current) - EMA(prev) ) x Multiplier) + EMA(prev)
				ema = previousEma + ((lTD.getClose() - previousEma) * factor);
//				Store EMA
				emaList.add(new Double(ema));
				previousEma = ema;
			} else {
				// Ignores
				//logger.debug("EMA ignored, null put in ArrayList");
				emaList.add(null);
			}
		}
		return emaList;
	}
	
	public static ArrayList computeEMA(ArrayList aList, int periode) {
		ArrayList emaList = new ArrayList();
		Double temp;
		double ema = 0.0;
		double previousEma = 0.0;
		double factor = (double)  2 / (periode +1);
		Double current;
		if (periode > aList.size() - 10) {
			// No meaningful calculation is possible
			return emaList;
		}
		int index = 0;
		for (int i = 0; i < periode;) {
			// Finds initial EMA (that is moving average for the first days)
			current = (Double) aList.get(index++);
			if (current != null) {
				previousEma = previousEma + current.doubleValue();
				i++;
			} 
			emaList.add(null);
		}
		previousEma = (double) previousEma / periode;
		// The first numberOfDays values are not correct
		for (int i = index; i < aList.size(); i++) {
			current = (Double) aList.get(i);
			if (current != null) {
				//( (Price(current) - EMA(prev) ) x Multiplier) + EMA(prev)
				ema = previousEma + ((current.doubleValue() - previousEma) * factor);
//				Store EMA
				emaList.add(new Double(ema));
				previousEma = ema;
			} else {
				// Ignores
				//logger.debug("EMA ignored, null put in ArrayList");
				emaList.add(null);
			}
		}
		return emaList;
	}
}
