/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/*
 * Date.java
 *
 * Created on 30 October 2001, 20:51
 */
package com.exponto.util;

import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 * Stores the date (year, month, day) and also has various methodes
 * for converting, finding working days and so one.
 * 
 * Basicly written before I got some understanding of the various 
 * SDK calendar and date classes 
 * 
 *  Konverterer en string p formatet yyyymmdd til
 * int year, month og day. yyyy m vre mellom 1700
 * og 2300, mm 1 til 12 og dd 1 til 31. Ingen sjekk 
 * p dd sammen med mm
 *
 * Bruker av og til JDK's greogorianske kalender, der
 * januar er 0, desember er 11. Sndag er 1 og lrdag
 * er 7.
 *
 * Kunne vrt noe bedre hndtering av feilformaterte 
 * strenger, eg brukt exceptions
 *
 * @author  kaare
 * @version 0.5
 */

public class Date {

	/**
	 * (English) Abbrivations for names of months
	 */
	public static final String [] monthsShort= { "jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec"};

    public int year, month, day = 0;
    public int week, dayInWeek = 0;
    private static Calendar kalender = new GregorianCalendar();
//    kalender.setFirstDayOfWeek(MONDAY);
    /** Creates new date */
    public Date() {
     }
     
    public Date(int y, int m, int d) {
    	year = y;
    	month = m;
    	day = d;
    }
    public Date(String s) {
        convert(s);
    }
    /** 
     * Creates a new Date with value (d - number of weeks)
     * A positive weekOffset gives an earlier date
     */
    public Date(Date d, int weekOffset) {
        kalender.set(d.year, d.month, d.day);
        kalender.add(Calendar.WEEK_OF_YEAR,-(weekOffset));
        year = kalender.get(Calendar.YEAR);
        month = kalender.get(Calendar.MONTH);
        day = kalender.get(Calendar.DAY_OF_MONTH);
        week = kalender.get(Calendar.WEEK_OF_YEAR);
        dayInWeek = kalender.get(Calendar.DAY_OF_WEEK);
        
    }
    public void convert (String s){
        if (s.length() == 8) {
            Integer t = new Integer(0);
            try {
                t = Integer.valueOf(s.substring(0, 4));
                year = t.intValue();
                t = Integer.valueOf(s.substring(4, 6)); 
                month = t.intValue()-1;
                t = Integer.valueOf(s.substring(6, 8));
                day = t.intValue();
            } catch (NumberFormatException e) {
                System.out.println("Klasse dato, metode convert:String is not a date = "+s);
            }
        } else System.out.println("Klasse dato, metode convert: String is too long or to short " + s); 
        int x = check();
        findWeekDay();
    }
    public int check () {
        // Fix me: Burde egentlig bruke noen exceptions her
        // Kan ogs bruke kalenderfunksjon 
        if (year < 1700 || year > 2300) {
        	
            System.out.println("Klasse dato, metode check: year out of range " + year);
            return 1;
        }
        if (month < 0 || month > 11) {
            System.out.println("Klasse dato, metode check: month out of range " + month);
            return 2;            
        }
        if (day < 1 || day > 31) {
            System.out.println("Klasse dato, metode check: day out of range " + day);
            return 3;
        }
        return 0;
    }
    public void findWeekDay () {
        kalender.set(year, month, day);
        week = kalender.get(Calendar.WEEK_OF_YEAR);
        dayInWeek = kalender.get(Calendar.DAY_OF_WEEK);
        //System.out.println("Uke: "+ week + " Dag i uke: " + dayInWeek+ " r: " + year + " Mned: " + month + " dag: " + day);
    }
    public void addDay() {
    	//Adds a day to current Date
    	// The kalender part automatically handles change of month, year etc. 
		kalender.set(this.year, this.month, this.day);
    	kalender.add(Calendar.DATE,1);
		this.year = kalender.get(Calendar.YEAR);
		this.month = kalender.get(Calendar.MONTH);
		this.day = kalender.get(Calendar.DAY_OF_MONTH);
		this.week = kalender.get(Calendar.WEEK_OF_YEAR);
		this.dayInWeek = kalender.get(Calendar.DAY_OF_WEEK);
    }
	public void setToNextWorkDay() {
		//Adds a day to current Date
		// The kalender part automatically handles change of month, year etc. 
		kalender.set(this.year, this.month, this.day);
		kalender.add(Calendar.DATE,1);
		this.dayInWeek = kalender.get(Calendar.DAY_OF_WEEK);
		while ((this.dayInWeek == 1)|| (this.dayInWeek == 7)) {
			kalender.add(Calendar.DATE,1);
			this.dayInWeek = kalender.get(Calendar.DAY_OF_WEEK);
		}
		this.year = kalender.get(Calendar.YEAR);
		this.month = kalender.get(Calendar.MONTH);
		this.day = kalender.get(Calendar.DAY_OF_MONTH);
		this.week = kalender.get(Calendar.WEEK_OF_YEAR);
		this.dayInWeek = kalender.get(Calendar.DAY_OF_WEEK);
	}    
   /**
    * Sets this day to the previous working day (that is mon -fri). Public holidays are
    * ignored.
    * 
	* The kalender part automatically handles change of month, year etc. 
    *
    */ 
	public void setToPreviousWorkDay() {
		kalender.set(this.year, this.month, this.day);
		kalender.add(Calendar.DATE,-1);
		this.dayInWeek = kalender.get(Calendar.DAY_OF_WEEK);
		while ((this.dayInWeek == 1)|| (this.dayInWeek == 7)) {
			// Next four line is porbably not needed
			this.year = kalender.get(Calendar.YEAR);
			this.month = kalender.get(Calendar.MONTH);
			this.day = kalender.get(Calendar.DAY_OF_MONTH);
			kalender.set(this.year, this.month, this.day);
			kalender.add(Calendar.DATE,-1);
			this.dayInWeek = kalender.get(Calendar.DAY_OF_WEEK);
		}
		this.year = kalender.get(Calendar.YEAR);
		this.month = kalender.get(Calendar.MONTH);
		this.day = kalender.get(Calendar.DAY_OF_MONTH);
		this.week = kalender.get(Calendar.WEEK_OF_YEAR);
//		this.dayInWeek = kalender.get(Calendar.DAY_OF_WEEK);
	}
	
/**
 * Adds (or subtracts) an amount of time (days, months or year) to the 
 * date.  
 * @param d	Date
 * @param field Which field to add
 * @param amount The amount to de/increase the field
 * @return new date
 */
	public static Date getNewDate(Date d, int field, int amount) {
		kalender.set(d.year, d.month, d.day); 
		kalender.roll(field, amount);
		Date newDate = new Date(kalender.get(Calendar.YEAR), kalender.get(Calendar.MONTH), kalender.get(Calendar.DAY_OF_MONTH));
		newDate.week = kalender.get(Calendar.WEEK_OF_YEAR);
		newDate.dayInWeek = kalender.get(Calendar.DAY_OF_WEEK);
		return newDate;
	}
	
    public Date klone () {
        Date cloned = new Date();
        cloned.year = this.year;
        cloned.month = this.month;
        cloned.day = this.day;
        cloned.week = this.week;
        cloned.dayInWeek = this.dayInWeek;
        return cloned;
    }
 /**
  * Compare this day with another day (year, month and day)
  * Returns 	0 if both dates are the same
  * 				-1/1 if year and month is equal, but date is different
  * 				-2/2 if year is equal but month is different
  * 				-4/4 if year is different
  * 				8 if total failure.
  * 
  * Negative values indicates that this date the lesser of the two
  *   
  *  
  * 
  * @param Date d to compare with 
  * @return int result of comparison
  */       
    public int compareDate(Date d) {
        if (this.year==d.year && this.month == d.month && this.day == d.day) return 0;
        if (this.year > d.year) return 4;
        if (this.year < d.year) return -4;
        if (this.month > d.month) return 2;
        if (this.month < d.month) return -2;
        if (this.year==d.year && this.month == d.month && this.day > d.day) return 1;
        if (this.year==d.year && this.month == d.month && this.day < d.day) return -1;
        return 8;
    }
    public int compareWeek(Date d) {
        // Returnerer 0 hvis uke og r er like, 
        // 2 eller -2 hvis samme r, men ukene er forskjellig
        // 4 eller -4 hvis forskjellig r
        // 8 hvis noe er galt
        if (this.year==d.year && this.week == d.week) return 0;
        if (this.year > d.year) return 4;
        if (this.year < d.year) return -4;
        if (this.week > d.week) return 2;
        if (this.week < d.week) return -2;
        return 8;
    }    
    public String toString() {
        String str = day + " " + monthsShort[month] + ". " + year;
        return str;
    }
	/**
	 * Returns the date as yyyymmdd
	 * @return the date as yyyymmdd
	 */
	public String toDigitString() {
		String s1, s2;
		int m = month + 1;
		if (m <= 9) {
			s1 = "0" + m;
		} else {
			s1 = String.valueOf(m);
		}
		if (day <= 9) {
			s2 = "0" + day;
		} else {
			s2 = String.valueOf(day);
		}		String str =  String.valueOf(year)  + s1 + s2;
		return str;
	}

    
//    public static void main(final String[] args) {
//        int x = 0;
//        Date d = new Date();
//		d.convert("16991231");
//		System.out.println(d.year + " "+ d.month + " " + d.day);
//        d.addDay();
//		System.out.println(" Neste dag:" + d.year + " "+ d.month + " " + d.day);
//		d.convert("20001131");
//		System.out.println(d.year + " "+ d.month + " " + d.day);
//		d.addDay();
//		System.out.println(" Neste dag:" + d.year + " "+ d.month + " " + d.day);
//		d.convert("20030630");
//		System.out.println(d.year + " "+ d.month + " " + d.day);
//		d.addDay();
//		System.out.println(" Neste dag:" + d.year + " "+ d.month + " " + d.day);

/*        // String too short
        d.convert("2001023");
        // String too long
        d.convert("200100239");          
        // Year not valid
        d.convert("16991123");
        if (d.check()==0)  System.out.println("OK");
        // Month not valid
        d.convert("20010023");
        if (d.check()==0)  System.out.println("OK");
        // Day not valid
        d.convert("20011100");
        if (d.check()==0)  System.out.println("OK");
        // OK
        d.convert("20011231");
        if (d.check()==0)  System.out.println("OK");
        System.out.println(d.toString());
        //System.out.println(d.year + " "+ d.month + " " + d.day);
        d.convert("20011323");
        if (d.check()==0)  System.out.println("OK"); 
        d.convert("20011132");
        if (d.check()==0)  System.out.println("OK");
        d.convert("87654321");
        if (d.check()==0)  System.out.println("OK"); */
//    }
}
