/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 18-Feb-2004
 *
 */
package com.exponto.util;

import java.awt.*;

import javax.swing.*;

/**
 * Layout an array of JComponents in a table like structure using
 * one or more JPanels with BoxLayout. 
 * 
 * @author kaare
 *
 */
public class TableLayoutPanel extends JPanel {

	//JPanel inputPanel;
	//JPanel infoPanel;
	//JLabel [] labelArray;
	/**
	 * 
	 */
	public TableLayoutPanel() {
		this.setLayout(new FlowLayout());
		this.setOpaque(true);
		
	}
	/**
	 * 
	 * The JComponent array contains the JComponents that this
	 * panel will include. the tableLayoutPanel will use one JPanel with the box
	 * layout for each column.  
	 * 
	 * The height of each component will be set to the highest component in 
	 * the first row. Preferred and min size will all be the same, while max size
	 * will be bigger.
	 * 
	 * A rigid area is inserted between the components in order to get some
	 * space between them.
	 * 
	 * A componentArray with the size [6][2] will have 2 columns and 6 rows.
	 * 
	 * Assumptions:
	 * 		1)No row will have more JComponents than the first
	 * 		2) No cell in the Array is null
	 */

	public TableLayoutPanel(JComponent [][] componentArray) {
		int columns;
		int rows;
		this.setLayout(new FlowLayout());
		this.setOpaque(true);
		double height = getPreferredHeight(componentArray);
//		Sizes are known here
		//System.out.println("Preferred size: " + componentArray[0][0].getPreferredSize().toString());
		//System.out.println("Max size: " + componentArray[0][0].getMaximumSize().toString());
		//System.out.println("Min size: " + componentArray[0][0].getMinimumSize().toString());

		//System.out.println("FontSize: " + fontSize);
		if (componentArray != null) {
			columns = componentArray[0].length;
			rows = componentArray.length;
		} else return;
		//Dimension dim = new Dimension();
		
		for (int c = 0; c < columns; c++ ) {
			
			JPanel jP = new JPanel();
			jP.setOpaque(true);
			jP.setLayout(new BoxLayout(jP, BoxLayout.Y_AXIS));
			jP.add(Box.createRigidArea(new Dimension(0,5)));
			for (int r = 0; r < rows; r++) {
				//System.out.println("r, c: " + r + ", " +c);
				//System.out.println("Component: " + componentArray[r][c].toString());
				//System.out.println("Width: " + componentArray[r][c].getPreferredSize().getWidth());
				// Keeps the width of the component, set new height
				Dimension dim = new Dimension();
				dim.setSize(componentArray[r][c].getPreferredSize().getWidth(), height);
				componentArray[r][c].setPreferredSize(dim);
				componentArray[r][c].setMinimumSize(dim);
				Dimension maxDim = new Dimension();
				maxDim.setSize((2 * componentArray[r][c].getPreferredSize().getWidth()), 2 * height);
				componentArray[r][c].setMaximumSize(maxDim);
				componentArray[r][c].setAlignmentX(Component.LEFT_ALIGNMENT);
				jP.add(componentArray[r][c]);
				jP.add(Box.createRigidArea(new Dimension(0,5)));
			}
			this.add(jP);
			
		}
	}

	private double getPreferredHeight(JComponent [][] componentArray) {
		int columns;
		double maxHeight = 0;
		if (componentArray != null) {
			columns = componentArray[0].length;
			double height;
			for (int c = 0; c < columns; c++ ) {
				height = componentArray[0][c].getPreferredSize().getHeight();
				if (height > maxHeight) {
					maxHeight= height;
				}
			}
		}
		return maxHeight;
	}

}
